from octologo.wizard import TextQuestion, SelectQuestion
from textual.validation import Number
from octologo.utils import font_list, color_scheme_names, FONTS_DIR, color_schemes, os, get_font_height, get_text_size
from PIL import Image, ImageDraw, ImageFont, ImageColor
import sys

sys.path.append("..")


questions = [
    SelectQuestion("font", "Select a font", [(font, font) for font in font_list], "Iosevka-Nerd-Font-Complete.ttf"),
    SelectQuestion("color", "Select a color scheme", color_scheme_names, "adi1090x"),
    TextQuestion("underline_count", "Lettrs to undrline", [Number(minimum=0)], "1", "1"),
    TextQuestion("padding_x", "Padding x (px)", [Number()], "200", "200"),
    TextQuestion("padding_y", "Padding y (px)", [Number()], "20", "20"),
    TextQuestion("gap", "Gap between text and bar (px)", [Number()], "20", "20"),
    TextQuestion("bar_size", "Bar weight (px)", [Number()], "20", "20"),
    TextQuestion("additionnal_bar_width", "Additionnal bar width (px)", [Number()], "20", "20"),
]

active = False


def get_image(answers):
    # Load the selected font
    font_size = 500
    font = ImageFont.truetype(os.path.join(FONTS_DIR, answers["font"]), font_size)

    # Set the colors
    background = ImageColor.getrgb(color_schemes[answers['color']]["background"])
    text = ImageColor.getrgb(color_schemes[answers['color']]["text"])
    accent = ImageColor.getrgb(color_schemes[answers['color']]["accent"])

    # Get the width and height of the texts
    text_width, text_height = get_text_size(answers['name'], font)
    font_height = get_font_height(font)

    # Get the correct image width and height
    image_width = 2 * int(answers['padding_x']) + text_width
    image_height = 2 * int(answers['padding_y']) + font_height

    # Create the image
    image = Image.new("RGB", (image_width, image_height), background)
    draw = ImageDraw.Draw(image)

    # Get the text anchor type and position on the image (where the text will be drawn)
    # LM = Left/Middle
    anchor_type = "lm"
    anchor_x = int(answers['padding_x'])
    anchor_y = image_height / 2 - (int(answers['gap']) + int(answers['bar_size'])) / 2

    anchor_pos = (anchor_x, anchor_y)

    if int(answers['underline_count']) > 0:
        # Get the bbox of the first n letter to underline
        first_letters_bbox = draw.textbbox(
            anchor_pos,
            answers['name'][:int(answers['underline_count'])],
            font=font,
            anchor=anchor_type)

        # Get the underline position
        underline_start_x = first_letters_bbox[0] - int(answers['additionnal_bar_width'])
        underline_start_y = first_letters_bbox[3] + int(answers['gap'])

        underline_end_x = int(answers['additionnal_bar_width']) + first_letters_bbox[2]

        underline_end_y = underline_start_y + int(answers['bar_size'])

        underline_start = (underline_start_x, underline_start_y)
        underline_end = (underline_end_x, underline_end_y)

        underline_pos = [underline_start, underline_end]

        # Underline the first letter
        draw.rectangle(underline_pos, fill=accent, width=answers['bar_size'])

    # Draw the text
    draw.text(
        anchor_pos,
        answers['name'],
        font=font,
        fill=text,
        anchor=anchor_type,
    )

    # Redraw the first letter
    draw.text(
        anchor_pos,
        answers['name'][0],
        font=font,
        fill=accent,
        anchor=anchor_type,
    )

    return image



from importlib import import_module
from typing import Any
import os
import toml
from PIL import Image, ImageDraw
from loguru import logger
from sys import stdout

logger.remove()
logger.add(
    stdout,
    format="[ <green>{time:HH:mm:ss}</green> ]"
    " - <level>{level}</level> -> "
    "<level>{message}</level>")


def get_text_size(text, font):
    text_bbox = ImageDraw.Draw(Image.new("RGBA", (1, 1), (0, 0, 0, 0))).textbbox(
        (0, 0), text, font=font
    )
    text_width = text_bbox[2] - text_bbox[0]
    text_height = text_bbox[3] - text_bbox[1]

    return text_width, text_height


def get_font_height(font):
    return font.getbbox("azertyuiopqsdfghjklmwxcvbnAZERTYUIOPQASDFGHJKLMWXCVBN0123456789")[3]


def remove_ext(filename):
    """
    Remove the extension from a filename if there is one
    """
    return filename.split(".")[0]


class Style():
    display_name: str
    module: Any

    def __init__(self, display_name: str, module: Any) -> None:
        self.display_name = display_name
        self.module = module


BASE_DIR = os.path.dirname(os.path.abspath(__file__))
FONTS_DIR = os.path.join(BASE_DIR, "fonts")
COLORS_DIR = os.path.join(BASE_DIR, "colors")


# Get all the fonts in the fonts directory
def get_font_list() -> list[str]:
    return os.listdir(FONTS_DIR)


font_list = get_font_list()


# Get all the color schemes in the colors directory
# keep only files with the .toml extension
# and remove the extension
def get_color_schemes() -> dict[str, dict[str, str]]:
    colors = dict()
    for color_file_name in os.listdir(COLORS_DIR):
        # Only keep .toml files
        if not color_file_name.endswith(".toml"):
            continue

        color_parsed = toml.load(os.path.join(COLORS_DIR, color_file_name))

        colors[remove_ext(color_file_name)] = color_parsed

    return colors


color_schemes = get_color_schemes()
color_scheme_names: dict[str, str] = [
    (color_schemes[color_scheme]['name'], color_scheme) for color_scheme in color_schemes]


def get_styles() -> dict[str, Style]:
    """
    return an array of style_id => Style {.module, .display_name}
    """
    # Load the styles in the styles directory
    styles = dict()
    for style in os.listdir(os.path.join(BASE_DIR, "styles")):
        # Only keep .py files
        if not style.endswith(".py"):
            continue

        module = import_module(f"octologo.styles.{remove_ext(style)}")

        # Only keep files with the active attribute set to True
        # This allows to ignore some scripts that may be in the styles directory
        if not module.active:
            continue

        styles[remove_ext(style)] = Style(module.display_name, module)

    return styles


styles = get_styles()

style_names: dict[str, str] = [(styles[style].display_name, style) for style in styles]
